﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Spline4
{
    class BasicFunctionClass
    {
        public const double _Pi = 3.14159265358979323846264338327950288419716939937510;

        public int nl = 20;
        public int TypeOfBasicElement = 5;

        public double sgn(double x)
        {
            if (x < 0.0) return -1.0;
            else if (x > 0.0) return 1.0;
            else return 0.0;
        }

        public double step(int n, double x)
        {
            if (n < 0) return 0.0;
            else
                switch (n)
                {
                    case 0: return 1.0;
                    case 1: return x;
                    default: return x * step(n - 1, x);
                }
        }

        public double Erf(double x)
        {
            double y, a1, a2, a3, a4, a5, p, t, z, r;
            p = 0.3275911; a1 = 0.254829592; a2 = -0.284496736;
            a3 = 1.421413741; a4 = -1.453152027; a5 = 1.061405429;
            y = Math.Abs(x); t = 1.0 / (1 + p * y);
            if (y > 10.0) z = 0.0;
            else z = Math.Exp(-y * y);
            r = 1.0 - (a1 * t + a2 * t * t + a3 * step(3, t) +
            a4 * step(4, t) + a5 * step(5, t)) * z;
            if (x < 0.0) r = -r;
            return r;
        }
        public double ErfB1(double x)
        {
            double p, q;
            p = x * x;
            if (p > 100.0) q = 0.0;
            else q = Math.Exp(-p) / Math.Sqrt(_Pi);
            return x * Erf(x) + q;
        }

        public double SplineNloc(double x)
        {
            double p, q, d, FN, FNlD;
            p = Math.Abs(x); d = Math.Sqrt(6.0); q = (p * p - d * d);
            if (p > 10.0) FN = 0.0;
            else FN = Math.Exp(-p * p / 2.0) / Math.Sqrt(2.0 * _Pi);
            if (p > d) FNlD = 0.0;
            else FNlD = FN * q * q * q;
            return FNlD;
        }

        public double ClocN(int nl)
        {
            int i; double a2, b2, d, h, s1, s2, y;
            d = Math.Sqrt(6.0); a2 = -d; b2 = d;
            h = (b2 - a2) / (2.0 * nl); s2 = 0.0; s1 = SplineNloc(b2 - h); y = a2;
            for (i = 1; i < nl; i++)
            {
                y = y + h; s1 = s1 + SplineNloc(y);
                y = y + h; s2 = s2 + SplineNloc(y);
            };
            return (SplineNloc(a2) + SplineNloc(b2) + 4.0 * s1 + 2.0 * s2) * h / 3.0;
        }

        public double SplinePolinomialB(int st, double x)
        {
            double ret = 0.0; double p = Math.Abs(x);
            if (st == 0)
            {
                if (p > 0.5) ret = 0.0;
                else ret = 1.0;
            }
            else
                if (st == 1)
                {
                    if (p > 1.0) ret = 0.0;
                    else ret = 1.0 - p;
                }
                else
                    if (st == 2)
                    {
                        if (p > 1.5) ret = 0.0;
                        else if (p > 0.5) ret = (p - 1.5) * (p - 1.5) / 2.0;
                        else ret = 0.75 - p * p;
                    }
                    else
                        if (st == 3)
                        {
                            double q = 1.0 - p;
                            double r = 2.0 - p;
                            if (p > 2.0) ret = 0.0;
                            else if (p > 1.0) ret = r * r * r / 6.0;
                            else ret = (1.0 + 3.0 * q + 3.0 * q * q - 3.0 * q * q * q) / 6.0;
                        }
            return ret;
        }
        public double SplineProectionalB(int st, double x)
        {
            double ret = 0.0;
            if (st == 0)
            {
                ret = (Erf((x + 0.5) / Math.Sqrt(2.0)) - Erf((x - 0.5) / Math.Sqrt(2.0))) / 2.0;
            }
            else
                if (st == 1)
                {
                    ret = (ErfB1((x + 1.0) / Math.Sqrt(2.0)) + ErfB1((x - 1.0) / Math.Sqrt(2.0)) -
                             2.0 * ErfB1(x / Math.Sqrt(2.0))) / Math.Sqrt(2.0);
                }
            return ret;
        }

        double Expp(double x)
        {
            return 1.0 / (1.0 + Math.Exp(-x)) - 0.5;
        }

        double Lnch(double x)
        {
            return Math.Log(2.0 * Math.Cosh(x / 2.0));
        }

        public double FunctionProectionalExp(double x)
        {
            return 2.0 * ((x + 1.0) * Expp(x + 1.0) + (x - 1.0) * Expp(x - 1.0) - 2.0 * x * Expp(x)) -
                    (Lnch(x + 1.0) + Lnch(x - 1.0) - 2.0 * Lnch(x)); ;
        }

        public double SplinePolinomialG(int st, double x)
        {
            double p = Math.Abs(x);
            return step(st, p);
        }

        public double SplineHiperbolic(int st, double al, double x)
        {
            double p = Math.Abs(x / al);
            double ret = 0.0;
            if (st == 1)
            {
                ret = Math.Sinh(p) / 2.0;
            }
            else
                if (st == 3)
                {
                    ret = (Math.Sinh(p) - p - p * p * p / 6.0) / 2.0 / step(4, al);
                }
            return ret;
        }

        public double SplineTrigonometric(int st, double al, double x)
        {
            double p = Math.Abs(x / al); double a = _Pi / 2.0 / al; double b = a / 2.0;
            double ret = 0.0;
            if (st == 3)
            {
                ret = a * (Math.Sin(p) - p * Math.Cos(p));
            }
            else
                if (st == 5)
                {
                    ret = (Math.Sin(p) - p * Math.Cos(p) + p * p / 3.0 * Math.Sin(p)) * b;
                }
            return ret;
        }

        public double SplineTrigonomExp(int st, double al, double x)
        {
            double p = Math.Abs(x / al); double q = (x / al);
            double ret = 0.0;
            if (st == 3)
            {
                ret = Math.Exp(-p) * (Math.Sin(p) + Math.Cos(p)) / 2.0 / al;
            }
            else
                if (st == 5)
                {
                    ret = sgn(x) * (Math.Cosh(q) * Math.Sin(q) - Math.Sinh(q) * Math.Cos(q)) / 4.0 / al;
                }
            return ret;
        }

        public double SplineOfStudent(int st, double al, double x)
        {
            double p = x / al;
            return 1.0 / step(st, 1.0 + p * p / (2.0 * st - 1.0));
        }

        public double SplineCorrelationalLag(int st, double al, double x)
        {
            double ret = 0.0; double p = Math.Abs(x / al);
            if (st == 1)
            {
                ret = 0.5 / al * Math.Exp(-p);
            }
            else
                if (st == 2)
                {
                    ret = 0.25 / al * (1.0 + p) * Math.Exp(-p);
                }
                else
                    if (st == 3)
                    {
                        ret = 3.0 / 16.0 / al * (1.0 + p + (1.0 / 3.0) * p * p) * Math.Exp(-p);
                    }
            return ret;
        }

        public double SplineCorrelationalLoc(int st, double al, double x)
        {
            double ret = 0.0; double p = Math.Abs(x / al); double a = 1.0 / (2.0 * (Math.Cosh(1.0) - 1.0));
            if (st == 1)
            {
                ret = (p < 1.0) ? a / al * Math.Sinh(1.0 - p) : 0.0;
            }
            else
                if (st == 2)
                {
                    double q = 1.0 - p;
                    double r = 2.0 - p;
                    if (p > 2.0) ret = 0.0;
                    else if (p > 1.0) ret = (r * Math.Cosh(r) - Math.Sinh(r)) / 2.0;
                    else ret = -Math.Cosh(p) + Math.Cosh(1.0) * (Math.Sinh(q) + p * Math.Cosh(q)) -
                        (Math.Sinh(p) - p * Math.Cosh(p)) / 2.0;
                }
            return a * a / al * ret;
        }

        public double SplineIntegral(int st, double al, double x)
        {
            double ret = 0.0; double p = Math.Abs(x); double q = Math.Abs(x / al);
            if (st == 2)
            {
                ret = (p < 0.5) ? (p * p + 0.25) : p;
            }
            else
                if (st == 3)
                {
                    ret = (q < 1.0) ? (-step(3, q) / 3.0 + step(2, q) + 1.0 / 3.0) : q;
                }
            return ret;
        }

        public double SplineOfVariableHardly(int st, double al, double x, double y)
        {
            double ret = 0.0;
            if (st == 1)
            {
                ret = (Math.Abs(x - y) / al * (Math.Exp(-x / al) + Math.Exp(-y / al)) / 2.0 + sgn(x - y) *
                             (Math.Exp(-x / al) - Math.Exp(-y / al))) * al;
            }
            else
                if (st == 2)
                {
                    ret = (Math.Abs(x - y) / al -
                        (1.0 + x / al * y / al) * sgn(x - y) * (Math.Atan(x / al) - Math.Atan(y / al))) * al / 4.0;
                }
            return ret;
        }
        public double FunctionGloc1(double x)
        {
            double p = Math.Abs(x);
            return (p < 1.0) ? 2.0 * step(3, p) - 3.0 * p * p + 1.0 : 0.0;
        }

        public double ClocG(int nl)
        {
            int i; double a2, b2, h, s1, s2, y;
            a2 = -1.0; b2 = 1.0;
            h = (b2 - a2) / (2.0 * nl); s2 = 0.0; s1 = FunctionGloc1(b2 - h); y = a2;
            for (i = 1; i < nl; i++)
            {
                y = y + h; s1 = s1 + FunctionGloc1(y);
                y = y + h; s2 = s2 + FunctionGloc1(y);
            };
            return (FunctionGloc1(a2) + FunctionGloc1(b2) + 4.0 * s1 + 2.0 * s2) * h / 3.0;
        }

        public double FunctionGloc2(double x)
        {
            double p = Math.Abs(x);
            return (p < 2.0) ? step(3, p) - 3.0 * p * p + 4.0 : 0.0;
        }

        public double FunctionLittleCap(double x)
        {
            double y = Math.Abs(x);
            return (y < 1) ? Math.Exp(-(1.0 / (1.0 - y * y))) : 0.0;
        }

        public double FunctionLocCos(int st, double x)
        {
            double ret = 0.0;
            double y = Math.Abs(x);
            if (st == 1)
                ret = (y < 1) ? step(st, Math.Cos(_Pi / 2.0 * x)) * step(st, 1.0 - x * x) : 0.0;
            else
                if (st == 2)
                    ret = (y < 1) ? step(st, Math.Cos(_Pi / 2.0 * x)) * step(st, 1.0 - x * x) : 0.0;
            return ret;
        }

        public double FunctionPolinomial(double x)
        {
            return x * x;
        }

        public double FunctionHiperbolic(double x)
        {
            return Math.Cosh(x) - 1.0;
        }

        public double BasicElement(int st, double al, double x, double y)
        {
            double ret = 0.0; double t = Math.Abs(x);
            switch (TypeOfBasicElement)
            {
                case 1: ret = SplinePolinomialB(0, x);
                    goto M;
                case 2: ret = SplinePolinomialB(1, x);
                    goto M;
                case 3: ret = SplineNloc(x) / ClocN(nl);
                    goto M;
                case 4: ret = FunctionGloc1(x) / ClocG(nl);
                    goto M;
                case 5: ret = SplinePolinomialG(st, x);
                    goto M;
                case 6: ret = SplinePolinomialG(3, Math.Sqrt(t));
                    goto M;
                case 7: ret = SplinePolinomialG(1, Math.Sqrt(t));
                    goto M;
                case 8: ret = SplinePolinomialB(st, x);
                    goto M;
                case 9: ret = SplineHiperbolic(st, al, x);
                    goto M;
                case 10: ret = SplineCorrelationalLag(st, al, x);
                    goto M;
                case 11: ret = SplineCorrelationalLoc(st, al, x);
                    goto M;
                case 12: ret = SplineTrigonometric(st, al, x);
                    goto M;
                case 13: ret = SplineTrigonomExp(st, al, x);
                    goto M;
                case 14: ret = SplineIntegral(st, al, x);
                    goto M;
                case 15: ret = SplineOfVariableHardly(st, al, x, y);
                    goto M;
                case 16: ret = SplineProectionalB(st, x);
                    goto M;
                case 17: ret = SplineOfStudent(st, al, x);
                    goto M;
                case 18: ret = FunctionProectionalExp(x);
                    goto M;
                case 19: ret = FunctionLittleCap(x);
                    goto M;
                case 20: ret = FunctionGloc2(x);
                    goto M;
                case 21: ret = FunctionLocCos(st, x);
                    goto M;
                case 22: ret = FunctionHiperbolic(x);
                    goto M;
                case 23: ret = FunctionPolinomial(x);
                    goto M;
                default: MessageBox.Show("sorry, method");
                    goto M;
            }
        M: return ret;
        }
    }
}
