﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using BaseLibrary;

namespace Spline5
{
    class BasisFunctionClass : BasicFunctionClass
    {
        //public int TypeOfBasicElement = 10;           //Методы: 1-10;
        public const double pi = 3.14159265358979323846264338327950288419716939937510;

        public const int N = 1001;
        static public int n = 101;

        public double[,] a = new double[N, N];
        public double[,] X = new double[N, N];

        public double h = (b1 - a1) / (double)(n - 1);

        static public int st = 2; //st = 1,2 
        public double al = 15.0;

        static public int keyOfJob = 1; //keyOfJob = 2; 
        public int KeyOfJob
        {
            get { return keyOfJob; }
            set { keyOfJob = value; }
        }

        static public double a1;
        public double Rangemin
        {
            get { return a1; }
            set { a1 = value; }
        }

        static public double b1;
        public double Rangemax
        {
            get { return b1; }
            set { b1 = value; }
        }

        public int Number
        {
            get { return n; }
            set { n = value; }
        }

        public double xi(int i)
        {
            double ret = 0.0;
                    ret = a1 + i * h;
            return ret;
        }


        public double KvasiLagrangeErf(int i, double x)
        {
            double p;
            p = (x - xi(i)) / h;
            return SplineProectionalB(0, p);
        }

        public double KvasiLagrangeErff(int i, double x)
        {
            double p;
            p = (x - xi(i)) / h;
            return SplineProectionalB(1, p);
        }

  
        public double LagrangeAlgebraic(int i, double x)
        {
            int k; double p,ri,rk;

            p = 1.0;
            if (keyOfJob == 1)
                ri = Math.Cos((2.0 * i + 1.0) * pi / (2.0 * n));
                        else ri = a1 + i * h;
            for (k = 0; k < n; k++)
            {
                if (keyOfJob == 1)
                    rk = Math.Cos((2.0 * k + 1.0) * pi / (2.0 * n));
                        else rk = a1 + k * h;
                if (k != i) p = p * (x - rk) / (ri - rk);
            };
            return p;
        }

        public double LagrangeTrigonometric(int i, double x)
        {
            int k; double p,rk,ri;

            p = 1.0;
            ri = 2.0 * i * pi / (2.0 * n + 1); 
            for (k = -n; k <= n; k++)
            {
                rk = 2.0 * k * pi / (2.0 * n + 1); 
                if (k != i) p = p * Math.Sin((x - rk) / 2.0) / Math.Sin((ri - rk) / 2.0);
            };
            return p;
        }

        public double LagrangeTrigonometricCos(int i, double x)
        {
            int k; double p,rk,ri;
            p = 1.0;
            ri = (2.0 * i + 1) * pi / 2.0 / (n + 1);
            for (k = 0; k <= n; k++)
            {
                rk = (2.0 * k + 1) * pi / 2.0 / (n + 1);
                if (k != i) p = p * (Math.Cos(x) - Math.Cos(rk)) / (Math.Cos(ri) - Math.Cos(rk));
            };
            return p;
        }

        public double LagrangeTrigonometricSin(int i, double x)
        {
            int k; double p,rk,ri;
            p = 1.0;
            ri = (i + 1) * pi / (n + 1);
            for (k = 0; k <= (n - 1); k++)
            {
                rk = (k + 1) * pi / (n + 1);
                if (k != i) p = p * (Math.Cos(x) - Math.Cos(rk)) / (Math.Cos(ri) - Math.Cos(rk));
            };
            p = p * Math.Sin(x) / Math.Sin(ri);
            return p;
        }

        public double KvasiLagrangeNDel(int i, double x)
        {
            double p, ND;
            p = (x - xi(i)) / h;
            if (p > 10.0) ND = 0.0;
            else ND = Math.Exp(-p * p / 2.0) / Math.Sqrt(2.0 * pi);
            return ND;
        }

        public double KvasiLagrangeLDel(int i, double x)
        {
            double p, LD;
            p = Math.Abs((x - xi(i)) / h);
            if (p > 50.0) LD = 0.0;
            else LD = (3.0 / 16.0) * (1.0 + p + p * p / 3.0) * Math.Exp(-p);
            return LD;
        }

        public double FNlocDel(double x)
        {
            double p, q, d, FN, FNlD;
            p = Math.Abs(x); d = Math.Sqrt(6.0); q = (p * p - d * d);
            if (p > 10.0) FN = 0.0;
            else FN = Math.Exp(-p * p / 2.0) / Math.Sqrt(2.0 * pi);
            if (p > d) FNlD = 0.0;
            else FNlD = FN * q * q * q;
            return FNlD;
        }

        public double Cloc(int nl)
        {
            int i; double a2, b2, d, h, s1, s2, y;
            d = Math.Sqrt(6.0); a2 = -d; b2 = d;
            h = (b2 - a2) / (2.0 * nl); s2 = 0.0; s1 = FNlocDel(b2 - h); y = a2;
            for (i = 1; i < nl; i++)
            {
                y = y + h; s1 = s1 + FNlocDel(y);
                y = y + h; s2 = s2 + FNlocDel(y);
            };
            return (FNlocDel(a2) + FNlocDel(b2) + 4.0 * s1 + 2.0 * s2) * h / 3.0;
        }

        public double KvasiLagrangeNlocDel(int i, double x)
        {
            double p;
            p = (x - xi(i)) / h;
            return 1.0 / Cloc(20) * FNlocDel(p);
        }

        public double BasicElementLoc(int st, double al, double x, double y)
        {
            double ret = 0.0;
            switch (TypeOfBasicElement)
            {
                case 9: ret = SplineCorrelationalLag(st, al, x);
                    goto M;
                case 10: ret = SplineCorrelationalLoc(st, al, x);
                    goto M;
                default: MessageBox.Show("sorry, bad method");
                    goto M;
            }
        M: return ret;
        }


        public double BasisFunction(int i, double x)
        {
            double ret = 0.0; double t = 0.0;
            switch (TypeOfBasicElement)
            {
                case 4: ret = KvasiLagrangeNDel(i, x);
                    break;
                case 5: ret = KvasiLagrangeLDel(i, x);
                    break;
                case 6: ret = KvasiLagrangeNlocDel(i, x);
                    break;
                case 7: ret = KvasiLagrangeErf(i, x);
                    break;
                case 8: ret = KvasiLagrangeErff(i, x);
                    break;
                case 9:
                    t = (x - xi(i));
                    ret = BasicElementLoc(st, al, t, 0.0);
                    break;
                case 10:
                    t = (x - xi(i)) / h;
                    ret = BasicElementLoc(st, al, t, 0.0);
                    break; 
                default: ret = 0.0;
                    MessageBox.Show("sorry 2, bad parameter TypeOfBasicElement");
                    break;
            }
            return ret;
        
        }


        public void MatrixVectf(double[,] a)//стр.137
        {
            int i, j;

            for (i = 0; i < n; i++)
                for (j = 0; j < n; j++)
                {
                    double x2 = a1 + j * h;
                    a[j, i] = BasisFunction(i, x2);
                };

            MessageBox.Show("MatrixVectf" + n.ToString());
            return;
        }

        public void MatrixInvers(double[,] a, double[,] X)
        {
            int i, j, k;
            for (k = 0; k < n; k++)
                for (i = 0; i < n; i++)
                    X[i, k] = (i == k) ? 1.0 : 0.0;
            for (k = 0; k < n; k++)
            {
                int l = k;
                double t = Math.Abs(a[k, k]);
                for (i = k + 1; i < n; i++)
                    if (Math.Abs(a[i, k]) > t)
                    {
                        l = i;
                        t = Math.Abs(a[i, k]);
                    }
                if (l != k)
                {
                    for (j = k; j < n; j++)
                    {
                        double t1 = a[k, j];
                        a[k, j] = a[l, j];
                        a[l, j] = t1;
                    };
                    for (j = 0; j < n; j++)
                    {
                        double t1 = X[k, j];
                        X[k, j] = X[l, j];
                        X[l, j] = t1;
                    };
                }
                for (j = k + 1; j < n; j++)
                    a[k, j] /= a[k, k];
                for (j = 0; j < n; j++)
                    X[k, j] /= a[k, k];

                for (i = 0; i < n; i++)
                    if (i != k)
                    {
                        for (j = k + 1; j < n; j++)
                            a[i, j] -= a[i, k] * a[k, j];
                        for (j = 0; j < n; j++)
                            X[i, j] -= a[i, k] * X[k, j];
                    }
            }

            MessageBox.Show("Gauss-Jordan n = " + n.ToString());
            return;
        }

        public double BasisFunctionInvers(int j, double x)//стр.137
        {
            int i; double s;
            s = 0.0;
            for (i = 0; i < n; i++)
                s = s + X[i, j] * BasisFunction(i, x);
            return s;
        }
    }
}
