﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using BaseLibrary;

namespace Spline5
{
    class InterpolationLocal : BasicFunctionClass
    {
        public const int ng = 1024;
 
        public int e = 0;
        public int[] xp = new int[ng];
        public int[] yp = new int[ng];
        public int[] zp = new int[ng];

        static public double ymas = 0.0;

        static public int numberGr = 1024;
        public int NumberGr
        {
            get { return numberGr; }
            set { numberGr = value; }
        }

        static public int[] xcoord;
        public int[] Xcoord
        {
            get { return xcoord; }
            set { xcoord = value; }
        }

        static public int[] ycoord;
        public int[] Ycoord
        {
            get { return ycoord; }
            set { ycoord = value; }
        }

        static public int[] zcoord;
        public int[] Zcoord
        {
            get { return zcoord; }
            set { zcoord = value; }
        }

        public double f(double x)
        {
            double ret = 0.0;
            switch (primer)
            {
                case 1: ret = 1.0 / (1.0 + 25.0 * x * x);
                    break;
                case 2: ret = 0.5 * x * x + 0.1 * Math.Sin(4.0 * _Pi * x);
                    break;
                case 3: ret = Math.Exp(4.0 * Math.Cos(4.0 * _Pi * x));
                    break;
                case 4: ret = Math.Sign(x) * step(2, x) * (x - a1) * (x - b1);
                    break;
                case 5: ret = Math.Abs(x);
                    break;
                case 6: ret = (Math.Abs(x) < 0.5) ? 1.0 : 0.0;
                    break;
                case 7: ret = (Math.Abs(x) < 0.5) ? sgn(x) : 0.0;
                    break;
                case 8: ret = Math.Exp(4.0 * Math.Sin(4.0 * _Pi * x));
                    break;
                case 9: ret = Erf(10.0 * x);
                    break;
                case 10: ret = -Math.Atan(Math.Cos(4.0 * _Pi * x)) + Math.Atan(1.0);
                    break;
                default:
                    MessageBox.Show("sorry 1, bad parameter primer");
                    break;
            }
            return ret;
        }

        public double fu(double x)
        {
            return (f(x) + f(-x))/2.0; 
        }

        public double fv(double x)
        {
            return (f(x) - f(-x))/2.0;
        }
 
        public double Ylin(double x)
        {
            return f(a1) + (f(b1) - f(a1)) / (b1 - a1) * (x - a1);
        }

        public double Ylinu(double x)
        {
            return fu(a1) + (fu(b1) - fu(a1)) / (b1 - a1) * (x - a1);
        }
        public double Ylinv(double x)
        {
            return fv(a1) + (fv(b1) - fv(a1)) / (b1 - a1) * (x - a1);
        }

        public double SummaInv(double x)
        {
            int i; double s;
            s = 0.0;
            for (i = 0; i < n; i++)
            {
                s = s + BasisFunctionInvers(i, x) * f(xi(i));
            }
            return s;
        }

        public double SummaDel(double x)
        {
            int i; double s,r;

            s = 0.0; r = 0.0;

            for (i = 0; i < n; i++)
            {
                r = xi(i);
                s = s + (f(r) - Ylin(r)) * BasisFunction(i, x);
           }
           return s + Ylin(x);
        }

        public double SummaDelAl(double x)
        {
            int i; double s, r;
            s = 0.0;
            for (i = 0; i < n; i++)
            {
                if (keyOfJob == 1)
                    r = Math.Cos((2.0 * i + 1.0) * _Pi / (2.0 * n));
                else 
                    r = xi(i);
                s = s + (f(r) - Ylin(r)) * LagrangeAlgebraic(i, x);
            };
            return s + Ylin(x);
        }
        
        public double SummaDeluv(double x)
        {
            int i; double s1,s2,r1,r2;

            s1 = 0.0;
            for (i = 0; i <= n; i++)
            {
                r1 = (2.0 * i + 1) * _Pi / 2.0 / (n + 1);
                s1 = s1 + (fu(r1) - Ylinu(r1)) * LagrangeTrigonometricCos(i, x);
            }
            s2 = 0.0;
            for (i = 0; i < n; i++)
            {
                r2 = (i + 1) * _Pi / (n + 1);
                s2 = s2 + (fv(r2) - Ylinv(r2)) * LagrangeTrigonometricSin(i, x);
            }
            return s1 + Ylinu(x) + s2 + Ylinv(x);
        }

        public double SummaDelTr(double x)
        {
            int i; double s = 0.0; double r;
            s = 0.0;

            for (i = -n; i <= n; i++)
            {
                r = 2.0 * i * _Pi / (2.0 * n + 1);
                s = s + (f(r) - Ylin(r)) * LagrangeTrigonometric(i, x); 
            }

            return s + Ylin(x);
        }

        public void Tabulation(int[] xp, int[] yp, int[] zp)
        {
            int k;
            double y, NormC, NormL2, s, h, yd, yr, zmax, ymax, zmin, ymin, xg, yg, zg;
            y = a1; NormC = -1.0; s = 0;
            ymas = -1.0;
            double z = (double)(ng - 1);
            h = (b1 - a1) / z;
            if (TypeOfBasicElement > 8)
            {
                MatrixVectf(a);
                MatrixInvers(a, X);
            }
            for (k = 0; k < ng; k++)
            {
                yg = f(y);
                if (TypeOfBasicElement == 1) 
                    zg = SummaDelAl(y);
                else
                    if (TypeOfBasicElement == 2) 
                        zg = SummaDelTr(y);
                    else
                        if (TypeOfBasicElement == 3) 
                            zg = SummaDeluv(y);
                        else
                            if (TypeOfBasicElement < 9) 
                                zg = SummaDel(y);
                            else
                                zg = SummaInv(y);
                if (Math.Abs(yg) > Math.Abs(zg))
                    yd = Math.Abs(yg);
                else
                    yd = Math.Abs(zg);
                if (yd > ymas)
                    ymas = yd;
                {
                    yr = Math.Abs(yg - zg);
                    if (yr > NormC)
                        NormC = yr;
                    s = s + yr * yr;
                }
                y = y + h;
            }

            ymax = ymas + 0.1 * ymas;
            ymin = -ymax;
            zmax = ymax;
            zmin = ymin;
            NormC = NormC / ymax; 

            y = a1;
            ymas = -1.0;
            for (k = 0; k < ng; k++)
            {
                xg = y;
                yg = f(y);
                if (TypeOfBasicElement == 1) 
                    zg = SummaDelAl(y);
                else
                    if (TypeOfBasicElement == 2) 
                        zg = SummaDelTr(y);
                    else
                        if (TypeOfBasicElement == 3) 
                            zg = SummaDeluv(y);
                        else
                            if (TypeOfBasicElement < 9) 
                                zg = SummaDel(y);
                            else
                                zg = SummaInv(y);
                xp[k] = (int)(1024 / (b1 - a1) * (xg - a1));
                yp[k] = 780 - (int)(780 / (ymax - ymin) * (yg - ymin));
                zp[k] = 780 - (int)(780 / (zmax - zmin) * (zg - zmin));
                y = y + h;
            }

            Number = n;
            Rangemin = a1;
            Rangemax = b1;
            NumberGr = ng;
            NormL2 = Math.Sqrt(s / z);
            Xcoord = xp;
            Ycoord = yp;
            Zcoord = zp;
            MessageBox.Show("NormC = : " + NormC.ToString());
            MessageBox.Show("NormL2 = : " + NormL2.ToString());
        }
    }
}
