﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using BaseLibrary;

namespace Spline8
{
    class EvolutionEquation : BasicFunctionClass
    {

        public const int ng = 1024;              
        public double alg = 3.0;//3.0-6.0;
        public double alc = 20.0;
        public double alh3 = 100.0;
        public double alh5 = 100.0;
        public double ali = 100.0;
        public double allocx = 4.0;     //4.0-10.0;
        public double allocr = 2.0;     //2.0-8.0;
        public double dx = 7.0;
        public double dr = 8.0;
        public double hl = 1.0;

        const int nt = 501;
        public double t0 = 0.0;
        public double tk = 3.0;
        public double t = 0.0;

        public int ni = 21;
        public int primer2 = 1; //1-9;
        static public int numberMethod2 = 1;//1,2,3,4,5,6,7,8

        static public double ymas = 0.0;

        static public int numberSlay = 1;// = methodSLAY;метод решения СЛАУ: 1-Gauss,2-Decomp-Solve

        public double[] b = new double[N];
        public double[] x = new double[N];
        public double[] c = new double[N];

        public int[] xp = new int[ng];
        public int[] yp = new int[ng];
        public int[] zp = new int[ng];

        static public int numberGr = 1024;
        public int NumberGr
        {
            get { return numberGr; }
            set { numberGr = value; }
        }

        public int NumberMethod2
        {
            get { return numberMethod2; }
            set { numberMethod2 = value; }
        }

        public int NumberSlay
        {
            get { return numberSlay; }
            set { numberSlay = value; }
        }

        static public int[] xcoord;
        public int[] Xcoord
        {
            get { return xcoord; }
            set { xcoord = value; }
        }

        static public int[] ycoord;
        public int[] Ycoord
        {
            get { return ycoord; }
            set { ycoord = value; }
        }

        static public int[] zcoord;
        public int[] Zcoord
        {
            get { return zcoord; }
            set { zcoord = value; }
        }

        //Локальные базисные функции
        public double SplineNloc2(double x)
        {
            double p, q, r;
            p = x * x;
            if (p > 100.0) q = 0.0; else q = Math.Exp(-p / 2) / Math.Sqrt(2 * _Pi);
            if (p > dx) r = 0.0; else r = q * step(3, p - dx);
            return r;
        }

        public double FiLocx(int i, double x)
        {
            double r, h1;
            double z = (double)(n - 1);
            h1 = (b1 - a1) / z; hl = h1 * Math.Sqrt(allocx);
            r = SplineNloc2((x - xi(i)) / hl);
            return r;
        }

        public double FiLocxp(int i, double x)
        {
            double p, q, ret, h1,t;
            double z = (double)(n - 1);
            h1 = (b1 - a1) / z;
            hl = h1 * Math.Sqrt(allocx);
            p = ((x - xi(i)) / hl) * ((x - xi(i)) / hl);
            q = p - dx; 
            if (p > 100.0) 
                t = 0.0; 
            else 
                t = Math.Exp(-p / 2) / Math.Sqrt(2 * _Pi);
            if (p > dx) 
                ret = 0.0;
            else 
                ret = t * q *
                    (q * (6.0 - q) - p * q * (6.0 - q) + 4.0 * p * (6 - q) - 2.0 * p * q);
            return ret;
        }

        public double FiH3x(int i, double x)
        {
            double ret, p, hl;
            double z = (double)(n - 1);
            double h = (b1 - a1) / z;
            hl = h * Math.Sqrt(alh5);
            p = (x - xi(i)) / hl;
            ret = sgn(p) * (Math.Sinh(p) - p - 1.0 / 6.0 * step(3, p) - 0.0 * 1.0 / 120.0 * step(5, p));
            return ret;
        }

        public double FiH3xp(int i, double x)
        {
            double ret, p, hl;
            double z = (double)(n - 1);
            double h = (b1 - a1) / z;
            hl = h * Math.Sqrt(alh5);
            p = (x - xi(i)) / hl;
            ret = sgn(p) * (Math.Sinh(p) - p - 0.0 * 1.0 / 6.0 * step(3, p));///hl/hl;
            return ret;
        }

        //Решения уравнения Пуассона        
        public double ff(double x, double y)
        {
            double ret = 0.0; double r = (x * x + y * y);
            switch (primer2)
            {
                case 1: ret = Math.Cos(1.5 * _Pi * x) * Math.Cos(1.5 * _Pi * y);
                    goto M;
                case 2: ret = 1.0 / (1.0 + 25.0 * (x * x + y * y));
                    goto M;
                case 3: ret = Math.Exp(Math.Cos(_Pi * r)) - 2 * r;
                    goto M;
                case 4: ret = Math.Exp(Math.Sin(2.0 * _Pi * r));
                    goto M;
                case 5: ret = Math.Exp(-r / 2.0 * 200.0);
                    goto M;
                case 6: ret = Math.Sin(5.0 / 2.0 * _Pi * x) * Math.Sin(5.0 / 2.0 * _Pi * y);
                    goto M;
                case 7: ret = Math.Sin(2.0 * _Pi * x) * Math.Sinh(2.0 * _Pi * y) / Math.Sinh(2.0 * _Pi);
                    goto M;
                case 8: ret = Math.Sin(2.0 * _Pi * x) * Math.Cosh(2.0 * _Pi * y) / Math.Cosh(2.0 * _Pi);
                    goto M;
                case 9: ret = Math.Sin(2.0 * _Pi * x) * Math.Exp(-2.0 * _Pi * y) / Math.Exp(2.0 * _Pi);
                    goto M;
                default: MessageBox.Show("sorry ff, bad parameter primer");
                    goto M;//break; 

            }
        M: return ret;
        }

        //Правая часть уравнения Пуассона
        public double fL(double x, double y)
        {
            double ret = 0.0; double r = (x * x + y * y);
            switch (primer2)
            {
                case 1: ret = -9.0 / 4.0 * _Pi * _Pi * (2.0 * Math.Cos(1.5 * _Pi * x) * Math.Cos(1.5 * _Pi * y));
                    goto M;
                case 2: ret = 100.0 / step(3, 1.0 + 25.0 * r) * (-1.0 + 25.0 * r);
                    goto M;
                case 3: ret = Math.Exp(Math.Cos(_Pi * r)) * (-4.0 * _Pi * Math.Sin(_Pi * r) -
                        4.0 * _Pi * _Pi * r * Math.Cos(_Pi * r) +
                        4.0 * _Pi * _Pi * r * Math.Sin(_Pi * r) * Math.Sin(_Pi * r)) - 8.0;
                    goto M;
                case 4: ret = Math.Exp(Math.Sin(2.0 * _Pi * r)) * (8.0 * _Pi * Math.Cos(2.0 * _Pi * r) -
                        16.0 * _Pi * _Pi * r * Math.Sin(2.0 * _Pi * r) +
                        16.0 * _Pi * _Pi * r * Math.Cos(2.0 * _Pi * r) * Math.Cos(2.0 * _Pi * r));
                    goto M;
                case 5: ret = Math.Exp(-r / 2.0 * 200.0) * (r * 200.0 - 2.0) * 200.0;
                    goto M;
                case 6: ret = -2.0 * (5.0 / 2.0) * (5.0 / 2.0) * _Pi * _Pi *
                    Math.Sin(5.0 / 2.0 * _Pi * x) * Math.Sin(5.0 / 2.0 * _Pi * y);
                    goto M;
                case 7: ret = 0.0;
                    goto M;
                case 8: ret = 0.0;
                    goto M;
                case 9: ret = 0.0;
                    goto M;
                default: MessageBox.Show("sorry ff, bad parameter primer");
                    goto M;

            }
        M: return ret;
        }

        // Двумерный базис для рашения уравнения Пуассона
        public double GG2(int i, int j, double x, double y)
        {
            double p;

            double ret = 0.0; double z = (double)(n - 1);
            double h = (b1 - a1) / z;
            switch (numberMethod2)
            {
                case 1: ret = step(5, Math.Abs(x - xi(i))) * step(5, Math.Abs(y - xi(j)));
                    goto M;
                case 2: ret = step(3, Math.Abs(x - xi(i))) * step(3, Math.Abs(y - xi(j)));
                    goto M;
                case 3: ret = FiH3x(i, x) * FiH3x(j, y);
                    goto M;
                case 4:
                    hl = h *Math.Sqrt(alg);
                    p = ((x - xi(i)) / hl) * ((x - xi(i)) / hl) + ((y - xi(j)) / hl) * ((y - xi(j)) / hl);
                    if (p > 100.0) 
                        ret = 0.0;
                    else
                        ret = Math.Exp(-p / 2);
                    goto M;
                case 5:
                    ret = FiLocx(i, x) * FiLocx(j, y);
                    goto M;         

                default: MessageBox.Show("sorry G, bad parameter Method");
                    goto M;
            }
        M: return ret;
        }

        // Двумерный базис после применения оператора Лапласа
        public double GGL(int i, int j, double x, double y)
        {
            double p;

            double ret = 0.0; double z = (double)(n - 1);
            double h = (b1 - a1) / z; 
            switch (numberMethod2)
            {
                case 1: ret = 20.0 * step(3, Math.Abs(x - xi(i))) * step(5, Math.Abs(y - xi(j))) +
                        20.0 * step(5, Math.Abs(x - xi(i))) * step(3, Math.Abs(y - xi(j)));
                    goto M;
                case 2: ret = 6.0 * step(1, Math.Abs(x - xi(i))) * step(3, Math.Abs(y - xi(j))) +
                        6.0 * step(3, Math.Abs(x - xi(i))) * step(1, Math.Abs(y - xi(j)));
                    goto M;
                case 3:
                    ret = FiH3xp(i, x) * FiH3x(j, y) + FiH3x(i, x) * FiH3xp(j, y);
                    goto M;
                case 4:
                    hl = h * Math.Sqrt(alg);
                    p = ((x - xi(i)) / hl) * ((x - xi(i)) / hl) + ((y - xi(j)) / hl) * ((y - xi(j)) / hl);
                    if (p > 100.0) ret = 0.0;
                    else
                        ret = Math.Exp(-p / 2) * (p - 2);// / h2;
                    goto M;
                case 5:
                    ret = (FiLocx(i, x) * FiLocxp(j, y) + FiLocxp(i, x) * FiLocx(j, y));// / h2;
                    goto M;
                default: MessageBox.Show("sorry G, bad parameter Method");
                    goto M;
            }
        M: return ret;
        }

        // Матрица-вектор в методе коллокаций. 
        //Их преобразование от четырехиндексного представления к двухиндексному
        //с последующим обращением к методам СЛАУ 
        public void MatrixVect2(double[,] a, double[] b)
        {
            int k, i, j, k1, i1, j1; bool log;
            double h2 = 1.0; 
            double z = (double)(n - 1);
            double h = (b1 - a1) / z;
                if (numberMethod2 == 3) 
                    h2 = h * h * alh3;
                else
                    if (numberMethod2 == 4) 
                        h2 = h * h * alg;
            k = 0;
            for (i = 0; i < n; i++)
                for (j = 0; j < n; j++)
                {
                    k1 = 0;
                    for (i1 = 0; i1 < n; i1++)
                        for (j1 = 0; j1 < n; j1++)
                        {
                            log = (i1 == 0) || (i1 == n - 1) || (j1 == 0) || (j1 == n - 1);
                            if (log) 
                                a[k1, k] = GG2(i, j, xi(i1), xi(j1));
                            else
                                a[k1, k] = GGL(i, j, xi(i1), xi(j1));
                            k1 = k1 + 1;
                        }
                    k = k + 1;
                }

            k = 0;
            for (i = 0; i < n; i++)
                for (j = 0; j < n; j++)
                {
                    log = (i == 0) || (i == n - 1) || (j == 0) || (j == n - 1);
                    if (log) b[k] = ff(xi(i), xi(j));
                    else
                        if (numberMethod2 > 2)
                            b[k] = fL(xi(i), xi(j))* h2; 
                        else 
                            b[k] = fL(xi(i), xi(j));  
                    k = k + 1;
                }

            MessageBox.Show("MatrixVectff " + n.ToString());
            return;
        }
  
        //Обратное преобразование двухиндексного представления 
        //матрицы и вектора в методе коллокаций к четырехиндексному.
        //Функция, осуществляющая доступ к матричным элементам матрицы a.
        public double aaZ(int k1, int k)
        {
            int j, j1; bool log;
            double ret;
            int i1 = (int)(k1 / n); j1 = k1 - i1 * n;
            int i = (int)(k / n); j = k - i * n;
            log = (i1 == 0) || (i1 == n - 1) || (j1 == 0) || (j1 == n - 1);
            if (log) 
                ret = GG2(i, j, xi(i1), xi(j1));
            else 
                ret = GGL(i, j, xi(i1), xi(j1));
            return ret;
        }

        //Функция, осуществляющая доступ к элементам вектора b.
        public double bbZ(int k)
        {
            int i, j; bool log;
            double ret;
            double h2 = 1.0;
            double z = (double)(n - 1);
            double h1 = (b1 - a1) / z;
            if (numberMethod2 == 5) 
                h2 = h1 * h1 * allocx;
            i = (int)(k / n); j = k - i * n;
            log = (i == 0) || (i == n - 1) || (j == 0) || (j == n - 1);
            if (log) 
                ret = ff(xi(i), xi(j));
            else
                ret = h2 * fL(xi(i), xi(j));
            return ret;
        }

        //Функция, осуществляющая вызов матричных элементов матрицы a 
        //и элементов вектора b из итерационного метода решения СЛАУ.
        public void MatrixVect2Z(int n, double[,] a, double[] b)
        {
            int k, k1;
            for (k = 0; k < n; k++)
                for (k1 = 0; k1 < n; k1++)
                {
                    a[k1, k] = aaZ(k1, k);
                }
            for (k = 0; k < n; k++)
                b[k] = bbZ(k);
            MessageBox.Show("MatrixVectff " + n.ToString());
            return;
        }

        //Функциия,определяющая приближенное решение уравнения Лапласа (базисы глобальны). 
        public double Summa2(double x1, double x2)
        {
            int k, i, j; double s;
            k = 0; s = 0.0;
            for (i = 0; i < n; i++)
                for (j = 0; j < n; j++)
                {
                    s = s + x[k] * GG2(i, j, x1, x2);
                    k = k + 1;
                }
            return s;
        }

        //Метод прямых для уравнения теплопроводности
        //Определение краеой задачи для обыкновенного дифференциального уравнения
        //второго порядка на текущем временном слое
        public double p1(double x)
        {
            double z = (double)(nt);
            double tau = (tk - t0) / z;
            double gamma = 1.0;
            double gamma5 = 1.0/(4 * _Pi * 4 * _Pi);
            double gamma6 = 1.0/(4.25 * _Pi * 4.25 * _Pi);
            double gamma7 = 1.0 / 25.0;
            double gamma8 = 1.0 / 25.0;
            double gamma9 = 1.0 / 3750.0;
            switch (primer2)
            {
                case 1: return -tau * gamma;
                case 2: return -tau * gamma; 
                case 3: return -tau * gamma; 
                case 4: return -tau * gamma; 
                case 5: return -tau * gamma5; 
                case 6: return -tau * gamma6;
                case 7: return -tau * gamma7;
                case 8: return -tau * gamma8;
                case 9: return -tau * gamma9; 

                default:
                    MessageBox.Show("sorry, primer");
                    return 0.0;
            }
        }

        public double q1(double x)
        {
            return 0.0;
        }

        public double r1(double x)
        {
            return 1.0;
        }

        public double f1(double x, double t)
        {
            double gamma9 = 1.0 / 3750.0;
            switch (primer2)
            {
                case 1: return 0.0;
                case 2: return 0.0;
                case 3: return 0.0;
                case 4: return 0.0;
                case 5: return 0.0;
                case 6: return 0.0;
                case 7: return 0.0;
                case 8: return 0.0;
                case 9: return Math.Exp(t) / (1.0 + 25.0 * x * x) -
                    gamma9 * Math.Exp(t) * (3750.0 * x * x - 50.0) / step(3, 1.0 + 25.0 * x * x);
                default:
                    MessageBox.Show("sorry, primer");
                    return 0.0;
            }
        }

        //Точное решение уравнения теплопроводности. 
        public double re1(double x,double t)
        {
            switch (primer2)
            {
                case 1: return Math.Exp(-t) * Math.Cos(x); 
                case 2: return Math.Exp(-t) * Math.Sin(x);
                case 3: return Math.Exp(t) * Math.Cosh(x) / Math.Exp(tk);
                case 4: return Math.Exp(t) * Math.Sinh(x) / Math.Exp(tk);
                case 5: return Math.Exp(-t) * Math.Cos(4 * _Pi * x);
                case 6: return Math.Exp(-t) * Math.Sin(4.25 * _Pi * x);
                case 7: return Math.Exp(t) * Math.Cosh(5.0 * x) / Math.Exp(tk);
                case 8: return Math.Exp(t) * Math.Sinh(5.0 * x) / Math.Exp(tk);
                case 9: return Math.Exp(t)/ (1.0 + 25.0 * x * x);

                default:
                    MessageBox.Show("sorry, primer");
                    return 0.0;
            }
        }

        //Базисные функции для представления решения краевой задачи
        public double GG1(double x, double y)
        {
            double h1, p, q, qf;
            p = (x - y);
            switch (NumberMethod2)
            {
                case 1: return sgn(p) * p * p * p;
                case 2: return sgn(p) * p * p * p * p * p;
                case 3: return sgn(p) * step(7, p);
                case 4: h1 = h * Math.Sqrt(alh5); q = (x - y) / h1;
                    return sgn(q) * (Math.Sinh(q) - q - 1.0 / 6.0 * step(3, q));
                case 5: h1 = h * Math.Sqrt(ali); q = (x - y) / h1; qf = Math.Abs(q);
                    return (qf < 1.0) ? 0.5 * (-step(3, qf) / 3.0 + step(2, qf) + 1.0 / 3.0) : 0.5 * qf;
                default:
                    MessageBox.Show("sorry, NumberMethod");
                    return 0.0;
            }
        }

        //Производные для базисных функций
        public double GG1p(double x, double y)
        {
            double h1, p, q, qf;
            p = (x - y); 
            switch (NumberMethod2)
            {
                case 1: return 3.0 * sgn(p) * p * p;
                case 2: return 5.0 * sgn(p) * p * p * p * p;
                case 3: return 7.0 * sgn(p) * step(6, p);
                case 4: h1 = h * Math.Sqrt(alh5); q = (x - y) / h1;
                    return sgn(q) * (Math.Cosh(q) - 1 - 0.5 * q * q) / h1;
                case 5: h1 = h * Math.Sqrt(ali); q = (x - y) / h1; qf = Math.Abs(q);
                    return (qf < 1.0) ? 0.5 * (-sgn(q) * q * q + 2.0 * q) / h1 : 0.5 * sgn(q) / h1;
                default:
                    MessageBox.Show("sorry, NumberMethod");
                    return 0.0;
            }
        }

        // Вторые производные для базисных функций
        public double GG1pp(double x, double y)
        {
            double h1, p, q, qf;
            p = (x - y); 
            switch (NumberMethod2)
            {
                case 1: return 6.0 * sgn(p) * p;
                case 2: return 20.0 * sgn(p) * p * p * p;
                case 3: return 42.0 * sgn(p) * step(5, p);
                case 4: h1 = h * Math.Sqrt(alh5); q = (x - y) / h1;
                    return sgn(q) * (Math.Sinh(q) - q) / h1 / h1;
                case 5: h1 = h * Math.Sqrt(ali); q = (x - y) / h1; qf = Math.Abs(q);
                    return (qf < 1.0) ? (-sgn(q) * q + 1.0) / h1 / h1 : 0.0;
                default:
                    MessageBox.Show("sorry, NumberMethod");
                    return 0.0;
            }
        }

        //Базисные функции
        public double FiGG1(int i, double x)
        {
            return GG1(x, a1 + i * h);
        }

        //Приближенное решение краеой задачи на текущем временном слое
        public double SummaGG1(double[] x, double y)
        {
            int i; double s; double z; double p;
            s = 0.0;
            for (i = 0; i < n; i++)
            {
                z = FiGG1(i, y);
                p = (numberSlay == 1) ? x[i] : b[i];
                s = s + z * p;
            }
            return s;
        }        

        //Вычисление матрицы-вектора в методе коллокаций для последующего применения СЛАУ
        //на текущем временном слое
        public void MatrixVectGG1(double t, double[,] a, double[] b)
        {
            double p = 0.0;
            int i, k;
            double z = (double)(nt);
            double tau = (tk - t0) / z;
            for (i = 0; i < n; i++)
                for (k = 1; k < (n - 1); k++)
                {
                    a[k, i] = p1(xi(k)) * GG1pp(xi(k), xi(i)) + q1(xi(k)) * GG1p(xi(k), xi(i)) +
                        r1(xi(k)) * GG1(xi(k), xi(i));
                }
            for (i = 0; i < n; i++)
            {
                a[0, i] = GG1(a1, xi(i));
            }
            for (i = 0; i < n; i++)
            {
                a[n - 1, i] = GG1(b1, xi(i));
            }
            for (k = 1; k < (n - 1); k++)
            {

                if (t < t0 + 1.5 * tau) p = re1(xi(k), t0);
                if (t > t0 + 1.5 * tau) p = SummaGG1(c, xi(k));
                b[k] = tau * f1(xi(k), t) + p; 
            }
            b[0] = re1(a1,t); b[n - 1] = re1(b1,t);
            return;
        }

        //Вычисление экранных координат
        public void Tabulation(int[] xp, int[] yp, int[] zp)
        {
            int k, i;
            double y, NormC, NormL2, s, h, yd, yr, zmax, ymax, zmin, ymin, yg, zg;
            NormC = -1.0; s = 0; zg = 0.0; yg = 0.0;
            ymas = -1.0;
            double z = (double)(ng - 1);
            h = (b1 - a1) / z;
            if (keyOfJob == 2)
            {
                double u = (double)(nt);
                double tau = (tk - t0) / u;
                for (i = 0; i < n; i++)
                    c[i] = 0.0;
                t = t0;
                for (k = 0; k < nt; k++)
                {
                    t = t + tau;
                    MatrixVectGG1(t, a, b);
                    Gauss(n, a, b, x);
                    for (i = 0; i < n; i++)
                        c[i] = x[i];
                };
            }
            y = a1;
            for (k = 0; k < ng; k++)
            {
                if (keyOfJob == 2)
                {
                    yg = re1(y, tk);
                    zg = SummaGG1(c, y);
                }
                else
                {
                    yg = ff(y, y);
                    zg = Summa2(y, y);
                }

                if (Math.Abs(yg) > Math.Abs(zg))
                    yd = Math.Abs(yg);
                else
                    yd = Math.Abs(zg);
                if (yd > ymas)
                    ymas = yd;
                yr = Math.Abs(yg - zg);
                if (yr > NormC)
                    NormC = yr;
                s = s + yr * yr;
                y = y + h;
            }

            ymax = ymas + 0.1 * ymas;
            ymin = -ymax;
            zmax = ymax;
            zmin = ymin;

            y = a1;
            for (k = 0; k < ng; k++)
            {
                if (keyOfJob == 2)
                {
                    yg = re1(y, tk);
                    zg = SummaGG1(c, y);
                }
                else
                {
                    yg = ff(y, y);
                    zg = Summa2(y, y);
                }
                xp[k] = (int)(1024 / (b1 - a1) * (y - a1));
                yp[k] = 780 - (int)(780 / (ymax - ymin) * (yg - ymin));
                zp[k] = 780 - (int)(780 / (zmax - zmin) * (zg - zmin));
                y = y + h;
            }

            Number = n;
            Rangemin = a1;
            Rangemax = b1;
            NumberGr = ng;
            NormL2 = Math.Sqrt(s / z);
            Xcoord = xp;
            Ycoord = yp;
            Zcoord = zp;
            MessageBox.Show("NormC = : " + NormC.ToString());
            MessageBox.Show("NormL2 = : " + NormL2.ToString());
        }
    }
}
